// Copyright (c) Mito
/*
  Contains a wrapper around a strongly typed object that simulates a web-api.
*/
// Max delay is the longest we'll wait for the API to return a value
const MAX_DELAY = 5000;
// How often we poll to see if we have a response yet
const RETRY_DELAY = 250;
const MAX_RETRIES = MAX_DELAY / RETRY_DELAY;
const getRandomId = () => {
    return '_' + Math.random().toString(36).substr(2, 9);
};
/*
  The MitoAPI class contains functions for interacting with the Mito backend. All
  interactions with the backend should go through this call, so that if we ever move
  to a more standard API, the migration is protected by this interface!
*/
export class MitoAPI {
    constructor(send) {
        this.send = send;
        this.unconsumedResponses = [];
    }
    /*
        The receive response function is a workaround to the fact that we _do not_ have
        a real API in practice. If/when we do have a real API, we'll get rid of this function,
        and allow the API to just make a call to a server, and wait on a response
    */
    receiveResponse(response) {
        this.unconsumedResponses.push(response);
    }
    /*
        Helper function that tries to get the response for a given ID, and returns
        the data inside the 'data' key in this response if it exists.

        Returns undefined if it does not get a response within the set timeframe
        for retries.
    */
    getResponseData(id, maxRetries = MAX_RETRIES) {
        return new Promise((resolve) => {
            let tries = 0;
            const interval = setInterval(() => {
                // Only try at most MAX_RETRIES times
                tries++;
                if (tries > maxRetries) {
                    console.log('Giving up on waiting');
                    clearInterval(interval);
                    // If we fail, we return an empty response
                    return resolve(undefined);
                }
                // See if there is an API response to this one specificially
                const index = this.unconsumedResponses.findIndex((response) => {
                    return response['id'] === id;
                });
                if (index !== -1) {
                    // Clear the interval
                    clearInterval(interval);
                    const response = this.unconsumedResponses[index];
                    this.unconsumedResponses.splice(index, 1);
                    return resolve(response['data']); // return to end execution
                }
                else {
                    console.log("Still waiting");
                }
            }, RETRY_DELAY);
        });
    }
    /*
        Returns all the CSV files in the current folder as the kernel.
    */
    async getDataFiles() {
        const id = getRandomId();
        this.send({
            'event': 'api_call',
            'type': 'datafiles',
            'id': id
        });
        const dataFiles = await this.getResponseData(id);
        if (dataFiles == undefined) {
            return [];
        }
        return dataFiles;
    }
    /*
        Returns a string encoding of the CSV file to download
    */
    async getDataframeAsCSV(sheetIndex) {
        const id = getRandomId();
        this.send({
            'event': 'api_call',
            'type': 'get_dataframe_as_csv',
            'id': id,
            'sheet_index': sheetIndex
        });
        // Note: We increase MAX_RETRIES to 250 although 100 worked locally for a dataset with 10M
        // rows and 4 columns, because the server is slower. 
        const sheetData = await this.getResponseData(id, 250);
        if (sheetData == undefined) {
            return '';
        }
        return sheetData;
    }
    /*
        Returns a string encoding of a PNG that can be displayed that
        is a summary graph of the specific column header at a specific
        sheet index. Optionally can pass a yAxis and yAxisSheetIndex
        if
    */
    async getGraph(xAxis, sheetIndexXAxis, yAxis, sheetIndexYAxis, height, width) {
        const id = getRandomId();
        this.send({
            'event': 'api_call',
            'type': 'get_graph',
            'id': id,
            'column_header_x_axis': xAxis,
            'sheet_index_x_axis': sheetIndexXAxis,
            'column_header_y_axis': yAxis,
            'sheet_index_y_axis': sheetIndexYAxis,
            'height': height,
            'width': width,
        });
        const graphString = await this.getResponseData(id, 250);
        if (graphString == undefined) {
            return undefined;
        }
        try {
            return JSON.parse(graphString);
        }
        catch (e) {
            return undefined;
        }
    }
    /*
        Returns a string of the dtype of a column
    */
    async getColumnDtype(sheetIndex, columnHeader) {
        const id = getRandomId();
        this.send({
            'event': 'api_call',
            'type': 'get_column_dtype',
            'id': id,
            'sheet_index': sheetIndex,
            'column_header': columnHeader
        });
        const dtype = await this.getResponseData(id);
        if (dtype == undefined) {
            return '';
        }
        return dtype;
    }
    /*
        Returns a list of the key, values that is returned by .describing
        this column
    */
    async getColumnDescribe(sheetIndex, columnHeader) {
        const id = getRandomId();
        this.send({
            'event': 'api_call',
            'type': 'get_column_describe',
            'id': id,
            'sheet_index': sheetIndex,
            'column_header': columnHeader
        });
        const describeString = await this.getResponseData(id, 100);
        if (describeString == undefined) {
            return {};
        }
        try {
            return JSON.parse(describeString);
        }
        catch (_a) {
            // We return nothing if we fail, as this is not a critical operation
            return {};
        }
    }
    /*
        Import summaries are a mapping from step_idx -> import information for each of the
        import steps in the analysis with the given analysisName.
    */
    async getImportSummary(analysisName) {
        const id = getRandomId();
        this.send({
            'event': 'api_call',
            'type': 'import_summary',
            'id': id,
            'analysis_name': analysisName
        });
        const importSumary = await this.getResponseData(id);
        if (importSumary == undefined) {
            return {};
        }
        return importSumary;
    }
    /*
        Adds a column with the passed parameters
    */
    async sendColumnAddMessage(sheetIndex, columnHeader, columnHeaderIndex, stepID) {
        if (stepID === undefined || stepID == '') {
            stepID = getRandomId();
        }
        this.send({
            'event': 'edit_event',
            'type': 'add_column_edit',
            'step_id': stepID,
            'sheet_index': sheetIndex,
            'column_header': columnHeader,
            'column_header_index': columnHeaderIndex
        });
        return stepID;
    }
    /*
        Adds a delete column message with the passed parameters
    */
    async sendDeleteColumn(sheetIndex, columnHeader) {
        const stepID = getRandomId();
        this.send({
            'event': 'edit_event',
            'type': 'delete_column_edit',
            'step_id': stepID,
            'sheet_index': sheetIndex,
            'column_header': columnHeader
        });
    }
    /*
        Does a merge with the passed parameters, returning the ID of the edit
        event that was generated (in case you want to overwrite it).
    */
    async sendMergeMessage(sheetOneIndex, sheetOneMergeKey, sheetOneSelectedColumns, sheetTwoIndex, sheetTwoMergeKey, sheetTwoSelectedColumns, 
    /*
        If you want to overwrite, you have to pass the ID of the the step that
        you want to overwrite. Not passing this argument, or passing an empty string,
        will result in no overwrite occuring (and a new stepID) being returned.
    */
    stepID) {
        // If this is overwriting a merge event, then we do not need to
        // create a new id, as we already have it!
        if (stepID === undefined || stepID == '') {
            stepID = getRandomId();
        }
        this.send({
            event: 'edit_event',
            type: 'merge_edit',
            step_id: stepID,
            sheet_index_one: sheetOneIndex,
            merge_key_one: sheetOneMergeKey,
            selected_columns_one: sheetOneSelectedColumns,
            sheet_index_two: sheetTwoIndex,
            merge_key_two: sheetTwoMergeKey,
            selected_columns_two: sheetTwoSelectedColumns,
        });
        return stepID;
    }
    /*
        Does a pivot with the passed parameters, returning the ID of the edit
        event that was generated (in case you want to overwrite it).
    */
    async sendPivotMessage(sheetIndex, pivotRows, pivotCols, values, stepID) {
        // If this is overwriting a pivot event, then we do not need to
        // create a new id, as we already have it!
        if (stepID === undefined || stepID === '') {
            stepID = getRandomId();
        }
        this.send({
            event: 'edit_event',
            type: 'pivot_edit',
            step_id: stepID,
            sheet_index: sheetIndex,
            pivot_rows: pivotRows,
            pivot_columns: pivotCols,
            values: values,
        });
        return stepID;
    }
    /*
        Reorders the columnHeader on sheetIndex to the newIndex, and shifts the remaining
        columns to the right.
    */
    async sendReorderColumnMessage(sheetIndex, columnHeader, newIndex) {
        const stepID = getRandomId();
        this.send({
            'event': 'edit_event',
            'type': 'reorder_column_edit',
            'step_id': stepID,
            'sheet_index': sheetIndex,
            'column_header': columnHeader,
            'new_column_index': newIndex
        });
    }
    /*
        Renames the dataframe at sheetIndex.
    */
    async sendDataframeRenameEdit(sheetIndex, newDataframeName, stepID) {
        // If this is overwriting a pivot event, then we do not need to
        // create a new id, as we already have it!
        if (stepID === undefined || stepID === '') {
            stepID = getRandomId();
        }
        this.send({
            'event': 'edit_event',
            'type': 'dataframe_rename_edit',
            'step_id': stepID,
            'sheet_index': sheetIndex,
            'new_dataframe_name': newDataframeName
        });
        return stepID;
    }
    /*
        Does a filter with the passed parameters, returning the ID of the edit
        event that was generated (in case you want to overwrite it).
    */
    async sendFilterMessage(sheetIndex, columnHeader, filters, operator, stepID) {
        // create a new id, if we need it!
        if (stepID === undefined || stepID === '') {
            stepID = getRandomId();
        }
        this.send({
            event: 'edit_event',
            type: 'filter_column_edit',
            step_id: stepID,
            sheet_index: sheetIndex,
            column_header: columnHeader,
            operator: operator,
            filters: filters,
        });
        return stepID;
    }
    /*
        Does a sort with the passed parameters, returning the ID of the edit
        event that was generated (in case you want to overwrite it).
    */
    async sendSortMessage(sheetIndex, columnHeader, sortDirection, stepID) {
        if (stepID === undefined || stepID === '') {
            stepID = getRandomId();
        }
        this.send({
            event: 'edit_event',
            type: 'sort_edit',
            step_id: stepID,
            sheet_index: sheetIndex,
            column_header: columnHeader,
            sort_direction: sortDirection,
        });
        return stepID;
    }
    /*
        Renames a column with the passed parameters
    */
    async sendRenameColumn(sheetIndex, oldColumnHeader, newColumnHeader, stepID) {
        if (stepID === undefined || stepID === '') {
            stepID = getRandomId();
        }
        this.send({
            event: 'edit_event',
            type: 'rename_column_edit',
            step_id: stepID,
            sheet_index: sheetIndex,
            old_column_header: oldColumnHeader,
            new_column_header: newColumnHeader,
        });
        return stepID;
    }
    /*
        Duplicates the dataframe at sheetIndex.
    */
    async sendDataframeDuplicateMessage(sheetIndex) {
        const stepID = getRandomId();
        this.send({
            'event': 'edit_event',
            'type': 'dataframe_duplicate_edit',
            'step_id': stepID,
            'sheet_index': sheetIndex,
        });
    }
    /*
        Deletes the dataframe at the passed sheetIndex
    */
    async sendDataframeDeleteMessage(sheetIndex) {
        const stepID = getRandomId();
        this.send({
            'event': 'edit_event',
            'type': 'dataframe_delete_edit',
            'step_id': stepID,
            'sheet_index': sheetIndex,
        });
    }
    /*
        Sends an undo message, which removes the last step that was created.
    */
    async sendUndoMessage() {
        this.send({
            'event': 'update_event',
            'type': 'undo'
        });
    }
    /*
        Sends a message which saves the current analysis with the passed
        analysisName.
    */
    async sendSaveAnalysisMessage(analysisName) {
        this.send({
            'event': 'update_event',
            'type': 'save_analysis_update',
            'analysis_name': analysisName
        });
    }
    /*
        Sends a message to tell Mito to replay an existing analysis onto
        the current analysis.
    */
    async sendUseExistingAnalysisUpdateMessage(analysisName, newFileNames, clearExistingAnalysis) {
        this.send({
            'event': 'update_event',
            'type': 'replay_analysis_update',
            'analysis_name': analysisName,
            'import_summaries': newFileNames === undefined ? {} : newFileNames,
            'clear_existing_analysis': clearExistingAnalysis === undefined ? false : clearExistingAnalysis
        });
    }
    /*
        Sends a message to tell Mito to delete a saved analysis.
    */
    async sendDeleteSavedAnalysisMessage(analysisName) {
        this.send({
            'event': 'update_event',
            'type': 'delete_analysis_update',
            'analysis_name': analysisName,
        });
    }
    /*
        Sends a message to tell Mito to rename a saved analysis.
    */
    async sendRenameSavedAnalysisMessage(old_analysis_name, new_analysis_name) {
        this.send({
            'event': 'update_event',
            'type': 'rename_analysis_update',
            'old_analysis_name': old_analysis_name,
            'new_analysis_name': new_analysis_name,
        });
    }
    /*
     Sends a message to get the description of a saved analysis
    */
    async getSavedAnalysisDescription(analysisName) {
        const id = getRandomId();
        this.send({
            'event': 'api_call',
            'type': 'get_saved_analysis_description',
            'id': id,
            'analysis_name': analysisName
        });
        const savedAnalysisDescription = await this.getResponseData(id);
        if (savedAnalysisDescription === undefined || savedAnalysisDescription === '') {
            return [];
        }
        return JSON.parse(savedAnalysisDescription);
    }
    /*
        Sets the formula for the given columns.
    */
    async sendSetColumnFormulaEditMessage(sheetIndex, columnHeader, newFormula) {
        const stepID = getRandomId();
        this.send({
            'event': 'edit_event',
            'type': 'set_column_formula_edit',
            'step_id': stepID,
            'sheet_index': sheetIndex,
            'column_header': columnHeader,
            'new_formula': newFormula
        });
    }
    /*
        Imports the given file names.
    */
    async sendSimpleImportMessage(fileNames, stepID) {
        if (stepID === undefined || stepID == '') {
            stepID = getRandomId();
        }
        this.send({
            'event': 'edit_event',
            'type': 'simple_import_edit',
            'step_id': stepID,
            'file_names': fileNames,
        });
        return stepID;
    }
    /*
        Does an import by running the pythonCode, and pulling out
        the new df names.
    */
    async sendRawPythonImportMessage(pythonCode, newDfNames) {
        const stepID = getRandomId();
        this.send({
            'event': 'edit_event',
            'type': 'raw_python_import_edit',
            'step_id': stepID,
            'python_code': pythonCode,
            'new_df_names': newDfNames,
        });
    }
    /*
        Sends the user_email to the backend so the user can sign in
    */
    async sendSignUp(userEmail) {
        this.send({
            'event': 'update_event',
            'type': 'set_user_field_update',
            'field': 'user_email',
            'value': userEmail
        });
    }
    /*
        Manually marks the tool as upgraded, which for now stops
        the upgrade modal from popping up multiple times a day
    */
    async manuallyMarkUpgraded() {
        this.send({
            'event': 'update_event',
            'type': 'set_user_field_update',
            'field': 'mitosheet_last_upgraded_date',
            // Taken from https://stackoverflow.com/questions/23593052/format-javascript-date-as-yyyy-mm-dd
            // We simply set the last upgraded date to today, which will delay showing the user another upgrade
            // message until they trigger the upgrade condition again
            'value': new Date().toISOString().split('T')[0]
        });
    }
    /*
        Checks outs a specific step by index
    */
    async checkoutStepByIndex(stepIndex) {
        this.send({
            'event': 'update_event',
            'type': 'checkout_step_by_idx_update',
            'step_idx': stepIndex
        });
    }
    /*
        If the user closes the feedback without giving any feedback,
        we call this, as we don't want to keep opening the feedback
        and annoying the user :)
    */
    async setClosedFeedback() {
        this.send({
            'event': 'update_event',
            'type': 'set_user_field_update',
            'field': 'closed_feedback',
            'value': true
        });
    }
    /*
        Tells the backend to mark the user as having gone through the tour in the user.json
    */
    async sendCloseTour(tourNames) {
        this.send({
            'event': 'update_event',
            'type': 'append_user_field_update',
            'field': 'received_tours',
            'value': tourNames
        });
    }
    /*
        Lets the backend know that new feedback was received. Note: the way
        we store the feedback in the backend is different than the way that
        we actually add params that get logged in this event.

        This is because we cannot easily get the answers if they are in nested
        objects, so we promote them to the highest level, so that we can easily
        make reports on them.
    */
    async sendFeedback(feedback) {
        const message = {
            'event': 'update_event',
            'type': 'append_user_field_update',
            'field': 'feedbacks',
            'value': feedback,
        };
        // We add the questions and answers to the highest level of the message
        // itself; see comment in send feedback
        let questionNum = 0;
        Object.keys(feedback).forEach(question => {
            message[`question_${questionNum}`] = question;
            message[`answer_${questionNum}`] = feedback[question];
            questionNum += 1;
        });
        this.send(message);
    }
    /*
        Sends a log event from the frontend to the backend, where it is logged
        by the backend. We log in the backend to keep a linear stream of actions
        that is making.
    */
    async sendLogMessage(logEventType, params) {
        let message = {};
        // Copy the params, so we don't accidently modify anything
        if (params !== undefined) {
            message = Object.assign({}, params);
        }
        // Get the browser information, so we can make sure
        // Mito works for all Mito users
        message['user_agent'] = window.navigator.userAgent;
        // Save the type of event, as well as what is being logged
        message['event'] = 'log_event';
        message['type'] = logEventType;
        this.send(message);
    }
}
//# sourceMappingURL=api.js.map