// Copyright (c) Mito
import React, { useEffect, useState } from 'react';
// import css
import '../../../css/graph-sidebar/graph-sidebar.css';
import '../../../css/graph-sidebar/loading-spinner.css';
import usePlotly from '../../hooks/usePlotly';
import XIcon from '../icons/XIcon';
import GraphAxis, { GraphAxisType } from './GraphAxis';
import GraphColumnSelector from './GraphColumnSelector';
import LoadingSpinner from './LoadingSpinner';
// Millisecond delay between loading graphs, so that
// we don't load to many graphs when the user is clicking around
const LOAD_GRAPH_TIMEOUT = 500;
/*
    This is the main component that displays all graphing
    functionality, allowing the user to build and view graphs.
*/
const GraphSidebar = (props) => {
    const [xAxis, setXAxis] = useState(undefined);
    const [sheetIndexXAxis, setSheetIndexXAxis] = useState(undefined);
    const [yAxis, setYAxis] = useState(undefined);
    const [sheetIndexYAxis, setSheetIndexYAxis] = useState(undefined);
    // We store each column selector seperatly b/c of race conditions around
    // closing the column selector when clicking off of it
    const [xAxisColumnSelectorOpen, setXAxisColumnSelectorOpen] = useState(false);
    const [yAxisColumnSelectorOpen, setYAxisColumnSelectorOpen] = useState(false);
    const [graphHTMLAndScript, setGraphHTMLAndScript] = useState(undefined);
    const [loading, setLoading] = useState(false);
    const [loadGraphTimeout, setLoadGraphTimeout] = useState(undefined);
    // We log when the graph has been opened
    useEffect(() => {
        void props.mitoAPI.sendLogMessage('opened_graph');
    }, []);
    // And also load the necessary plotly libraries
    usePlotly();
    /*
        Whenever the axes are changed, we set a timeout to start loading a new
        graph. This runs after LOAD_GRAPH_TIMEOUT.

        This makes sure we don't send unnecessary messages to the backend if the user
        is switching axes quickly.
    */
    useEffect(() => {
        if (loadGraphTimeout !== undefined) {
            clearTimeout(loadGraphTimeout);
        }
        const newLoadGraphTimeout = setTimeout(loadNewGraph, LOAD_GRAPH_TIMEOUT);
        setLoadGraphTimeout(newLoadGraphTimeout);
        // We also log when the columns are selected, when they change
        void props.mitoAPI.sendLogMessage('graph_selected_column_changed', {
            'generated_graph': xAxis !== undefined,
            'column_header_x_axis': xAxis,
            'sheet_index_x_axis': sheetIndexXAxis,
            'column_header_y_axis': yAxis,
            'sheet_index_y_axis': sheetIndexYAxis,
        });
    }, [xAxis, yAxis]);
    // When we get a new graph script, we execute it here. This is a workaround
    // that is required because we need to make sure this code runs, which it does
    // not when it is a script tag inside innerHtml (which react does not execute
    // for safety reasons).
    useEffect(() => {
        try {
            if (graphHTMLAndScript === undefined) {
                return;
            }
            const executeScript = new Function(graphHTMLAndScript.script);
            executeScript();
        }
        catch (e) {
            console.error("Failed to execute graph function", e);
        }
    }, [graphHTMLAndScript]);
    /*
        This is the actual function responsible for loading the new
        graph from the backend, making sure this graph is the correct
        size.
    */
    const getGraphAsync = async () => {
        var _a;
        setLoading(true);
        const boundingRect = (_a = document.getElementById('graph-div')) === null || _a === void 0 ? void 0 : _a.getBoundingClientRect();
        const loadedGraphHTMLAndScript = await props.mitoAPI.getGraph(xAxis, sheetIndexXAxis, yAxis, sheetIndexYAxis, `${boundingRect === null || boundingRect === void 0 ? void 0 : boundingRect.height}px`, `${boundingRect === null || boundingRect === void 0 ? void 0 : boundingRect.width}px`);
        setGraphHTMLAndScript(loadedGraphHTMLAndScript);
        setLoading(false);
    };
    const loadNewGraph = async () => {
        void getGraphAsync();
    };
    const setAxis = async (axis, columnHeader) => {
        var _a, _b;
        if (axis === GraphAxisType.X_AXIS) {
            setSheetIndexXAxis(props.selectedSheetIndex);
            setXAxis(columnHeader);
        }
        else if (axis === GraphAxisType.Y_AXIS) {
            setSheetIndexYAxis(props.selectedSheetIndex);
            setYAxis(columnHeader);
        }
        // Select the column and scroll it into view
        props.setSelectedColumn(columnHeader);
        (_b = (_a = window.gridApiMap) === null || _a === void 0 ? void 0 : _a.get(props.model_id)) === null || _b === void 0 ? void 0 : _b.ensureColumnVisible(columnHeader);
    };
    if (props.sheetJSONArray.length === 0) {
        return (React.createElement("div", { className: 'graph-sidebar-div' },
            React.createElement("div", { className: 'graph-sidebar-no-data-div' },
                React.createElement("p", { className: 'graph-sidebar-no-data-text' }, "You need to import data into Mito before graphing!"))));
    }
    else {
        return (React.createElement("div", { className: 'graph-sidebar-div' },
            React.createElement("div", { className: 'graph-sidebar-y-axis-div' },
                React.createElement(GraphAxis, { axis: GraphAxisType.Y_AXIS, columnHeader: yAxis, openColumnSelector: () => setYAxisColumnSelectorOpen(true) })),
            React.createElement("div", { className: 'graph-sidebar-center-column-div' },
                React.createElement("div", { className: 'graph-sidebar-graph-div', id: 'graph-div' },
                    graphHTMLAndScript === undefined && xAxis === undefined && yAxis === undefined &&
                        React.createElement("p", { className: 'graph-sidebar-welcome-text' }, "To generate a graph, select a axis."),
                    graphHTMLAndScript !== undefined &&
                        React.createElement("div", { className: 'graph-sidebar-graph', dangerouslySetInnerHTML: { __html: graphHTMLAndScript === null || graphHTMLAndScript === void 0 ? void 0 : graphHTMLAndScript.html } })),
                React.createElement(GraphAxis, { axis: GraphAxisType.X_AXIS, columnHeader: xAxis, openColumnSelector: () => setXAxisColumnSelectorOpen(true) })),
            React.createElement("div", { className: 'graph-sidebar-toolbar-div', onClick: () => {
                    props.setGraphSidebarOpen(false);
                    void props.mitoAPI.sendLogMessage('closed_graph');
                } },
                React.createElement(XIcon, null)),
            xAxisColumnSelectorOpen &&
                React.createElement(GraphColumnSelector, { axis: GraphAxisType.X_AXIS, selectedColumnHeader: xAxis, columnHeaders: props.sheetJSONArray[props.selectedSheetIndex].columns, columnTypes: props.columnTypes, setAxis: setAxis, closeColumnSelector: () => setXAxisColumnSelectorOpen(false) }),
            !xAxisColumnSelectorOpen && yAxisColumnSelectorOpen &&
                React.createElement(GraphColumnSelector, { axis: GraphAxisType.Y_AXIS, selectedColumnHeader: yAxis, columnHeaders: props.sheetJSONArray[props.selectedSheetIndex].columns, columnTypes: props.columnTypes, setAxis: setAxis, closeColumnSelector: () => setYAxisColumnSelectorOpen(false) }),
            loading &&
                React.createElement("div", { className: 'loading-spinner-div' },
                    React.createElement(LoadingSpinner, null),
                    React.createElement("p", { className: 'loading-spinner-div-text' }, "loading"))));
    }
};
export default GraphSidebar;
//# sourceMappingURL=GraphSidebar.js.map